/***************************************************************************
                          ktritoc  -  a KDE game ai.cpp
                             -------------------
    begin                : November 2002
    copyright            : (C) 2002 by M. Heni
    email                : ktritoc@heni-online.de
  ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef USEKDE
#include <kdebug.h>
#include <stdlib.h>
#endif
//#include <stream.h>

#include "ai.h"
#include "ktritocdoc.h"
#include "dice.h"
#include "gameboard.h"

#define ACTION_NONE   0  // Do nothing
# define ACTION_DICE   1  // throw dice
# define ACTION_MOVE 2  // Make move


// AI Value settings
#define FIELD_WON      10000  // Bonus for won board
#define FIELD_DRAW         0  // Bonus for drawn board
#define FIELD_ROW_2     1500  // Bonus for row of len = 2
#define FIELD_CENTER     900  // center bonus
#define FIELD_CORNER     200  // Corner bonus
#define BOARD_MULTIPLY    10  // Board more important than field
#define MIN_VALUE      -999999999L // For min max search

#define AGGRESSIVENESS   1.2   // >1.0 prefer attack versus defense

Ai::Ai(KtritocDoc *doc) : QObject(0,"Ai")
{
  mDoc = doc;
  mPlayer = 0;
  mAction = ACTION_NONE;
  mTimer = new QTimer(this);
  mTimer->stop();
  connect(mTimer,SIGNAL(timeout()),
                this,SLOT(action()));

  // 8 possible combinations to win
  //  0  1  2
  //  3  4  5
  //  6  7  8
  mLines.resize(8);
  mLines[0].a = 0; mLines[0].b = 1; mLines[0].c = 2;
  mLines[1].a = 3; mLines[1].b = 4; mLines[1].c = 5;
  mLines[2].a = 6; mLines[2].b = 7; mLines[2].c = 8;
  mLines[3].a = 0; mLines[3].b = 3; mLines[3].c = 6;
  mLines[4].a = 1; mLines[4].b = 4; mLines[4].c = 7;
  mLines[5].a = 2; mLines[5].b = 5; mLines[5].c = 8;
  mLines[6].a = 0; mLines[6].b = 4; mLines[6].c = 8;
  mLines[7].a = 2; mLines[7].b = 4; mLines[7].c = 6;
}

Ai::~Ai()
{
  if (mTimer) delete mTimer;
}

void Ai::stop()
{
  if (mTimer) mTimer->stop();
}

/** Let the Ai throw the dice */
void Ai::throwDice(){
    mAction = ACTION_DICE;
    mTimer->start(500,true);
}

/** Let the Ai  make a move */
void Ai::makeMove(int player){
    mAction = ACTION_MOVE;
    mPlayer=player;
    mTimer->start(500,true);
}

/** Perform AI action */
void Ai::action()
{
  switch(mAction)
  {
     case ACTION_DICE:
       #ifdef USEKDE
       kdDebug() << "Ai throws dice " << endl;
       #endif
       mDoc->getDice()->throwit();
     break;
     case ACTION_MOVE:
       #ifdef USEKDE
       kdDebug() << "Ai makes move for "<<mPlayer << endl;
       #endif
       searchMove();
     break;
     default:
       #ifdef USEKDE
       kdDebug() << "No AI action " << endl;
       #endif
     break;
  }
}

void Ai::searchMove()
{
  int dicevalue = mDoc->getDice()->value();
  int i,d;
  int x,y;

#ifdef ABC  
  for (i=0;i<9;i++)
  {
    if (!mDoc->getBoard(i)->getXYFromDice(dicevalue,x,y)) continue;
    if (0==mDoc->getBoard(i)->getField(x,y))
    {
      int no = mDoc->getBoard(i)->setField(x,y,mPlayer);
      mDoc->finishMove(i,no);
      return ;
    }
  }
#endif

  int opponent = 3-mPlayer;

  QMemArray<int> board;
  board.resize(9);
  for (i=0;i<9;i++) board[i] = mDoc->getBoard(i)->getOwner();

  QMemArray<int> field;
  field.resize(9);
  QMemArray<int> board_value;
  board_value.resize(9);

  // New: Determine criticality of boards
  // First loop boards
  // cout << "----------------------"<<endl;
  for (i=0;i<9;i++)
  {
    int b_winner;
    int b_value;
    int a1,a2;
    board_value[i] = 0;
    if (board[i]!=0) continue;
    board[i] = mPlayer;
    b_value = calc_value(board,mPlayer,b_winner,false);
    a1=b_value;
    board_value[i] += (int)( (double)b_value*AGGRESSIVENESS );
    board[i] = opponent;
    b_value = calc_value(board,opponent,b_winner,false);
    a2=b_value;
    board_value[i] += b_value;
    // cout << "board["<<i<<"] = "<< board_value[i]<< "("<<a1<<","<<a2<<")"<< endl;
    board[i] = 0;
  }

  // Search data
  int max_value = MIN_VALUE; // Score of best move
  int max_board = -1;
  int max_dice = -1;

  // Loop all possible moves
  // First loop boards
  for (i=0;i<9;i++)
  {
    // Do not try full boards
    if (board[i]!=0) continue;

    // Copy current field to array
    for (int j=0;j<9;j++) field[j]=mDoc->getBoard(i)->getFieldFromNumber(j);


    // Now loop all fields (dice)
    for (d=2;d<=12;d++)
    {
      long value = MIN_VALUE;

      int dice_field;
      // Get index in array from dice
      if (!mDoc->getBoard(i)->getNumberFromDice(d,dice_field)) continue;

      // Check dice 2,12,number?
      if (dicevalue == 12 || dicevalue == 2)
      {
        if (field[dice_field]==0) continue; // field empty!
        if (dicevalue == 12 && field[dice_field]==mPlayer) continue; // own piece 
        if (dicevalue == 2 && field[dice_field]==opponent) continue; // not own piece 

        int cur_color = field[dice_field]; // store current color
        // int cur_opponent = 3-cur_color;

        // Local variables
        int f_value;
        int b_value;
        int f_winner;
        int b_winner;

        // Reset current value
        value = 0;

        // Remove piece and see what happens
        field[dice_field] = 0;
        f_value = calc_value(field,cur_color,f_winner,false);
        if (f_winner>0) board[i] = f_winner;  // Board is just won
        b_value = calc_value(board,cur_color,b_winner,false);
        // Sub score for opponent's loss
        value -= f_value + b_value * BOARD_MULTIPLY;

        // Reset board 
        board[i] = 0;

        // Get base value (as it is now!)
        field[dice_field] = cur_color;
        f_value = calc_value(field,cur_color,f_winner,false);
        if (f_winner>0) board[i] = f_winner;  // Board is just won
        b_value = calc_value(board,cur_color,b_winner,false);
        // Add value
        value += f_value + b_value * BOARD_MULTIPLY;

        // Reset board 
        board[i] = 0;

        // We calucluated it for a 12 for a 2 we have to reverse it
        if (dicevalue == 2) value = -value;

        #ifdef USEKDE
        kdDebug() << "AI checks REMOVE board " << i << "dice " << d << " value="<<value<<endl;
        #endif
      }
      else if (dicevalue != d)  // normal number non matching
      {
        continue;
      }
      else if (dice_field>=0)  // Dice has corresponding field
      {
        if (field[dice_field]!=0) continue; // field occupied!

        // Local variables
        int f_value;
        int b_value;
        int f_winner;
        int b_winner;

        // Reset current value
        value = 0;

        // Make move player!
        field[dice_field] = mPlayer;
        f_value = calc_value(field,mPlayer,f_winner,false);
        if (f_winner>0) board[i] = f_winner;  // Board is just won
        b_value = calc_value(board,mPlayer,b_winner,false);
        // Add score
        value += (long)((f_value + b_value * BOARD_MULTIPLY)*AGGRESSIVENESS) ;

        // Reset board 
        board[i] = 0;

        // Make move opponent
        field[dice_field] = opponent;
        f_value = calc_value(field,opponent,f_winner,false);
        if (f_winner>0) board[i] = f_winner;  // Board is just won
        b_value = calc_value(board,opponent,b_winner,false);
        // Add score
        value += f_value + b_value * BOARD_MULTIPLY;

        // Reset board 
        board[i] = 0;

        // Get base value (as it is now!)
        field[dice_field] = 0;
        f_value = calc_value(field,mPlayer,f_winner,false);
        if (f_winner>0) board[i] = f_winner;  // Board is just won
        b_value = calc_value(board,mPlayer,b_winner,false);
        // Subtract two times base score
        value -= (long)((f_value + b_value * BOARD_MULTIPLY)*AGGRESSIVENESS);
        value -= f_value + b_value * BOARD_MULTIPLY;

        // Reset board 
        board[i] = 0;

        #ifdef USEKDE
        kdDebug() << "AI checks board " << i << "dice " << d << " value="<<value<<endl;
        #endif
      }
      else // number matching but no dice field ... error
      {
        break;
      }

      // Found a good move --------- !!!!
      // New add general board value to get better global overview
      value += board_value[i];

      // cout << "Value(b="<<i<<",f="<<d<<") =>"<<value-board_value[i]<<"=>"<<value<<endl;

      if (value>max_value)
      {
        max_value = value;
        max_dice = d;
        max_board = i;
      }
    }// next dice
  }// next board
  

  int color = mPlayer; // This player should move
  // In case of removal we set the color to 0. This
  // is the case if the dice value is not the played
  // dice value 
  if (dicevalue != max_dice) color = 0;

  #ifdef USEKDE
  kdDebug() << "AI chooses board " << max_board << "dice " << max_dice << " value="<<max_value<<endl;
  #endif
  if (max_dice<0 || max_board<0)
  {
   #ifdef USEKDE
   kdError() << "AI ERROR " << endl;
   exit(1);
   #endif
   return ;
  }
  // Perform move - it should be a legal move if the AI
  // Calculated it right beforehand...
  mDoc->getBoard(max_board)->getXYFromDice(max_dice,x,y);
  int no = mDoc->getBoard(max_board)->setField(x,y,color);
  mDoc->finishMove(max_board,no );
}

/**
 * field: memarray containing the piece owners (0,1,2)
 * player = 1,2
 * check_won: search only for board win or drawn/nothing
 * Returns: value for the field (+:good for player, -:not good)
 */
int Ai::calc_value(QMemArray<int> &field,int player, int& winner, bool check_won)
{
//  0  1  2
//  3  4  5
//  6  7  8

  int i;
  int opponent = 3-player;

  // How many occupied fields
  int occupied = 0;
  for (i=0;i<9;i++) occupied += (field[i]!=0 ? 1:0);

  // Check for winning tupels
  winner = 0;
  for (i=0;i<(int)mLines.count();i++)
  {
    if ( (field[mLines[i].a] != 0) &&
         (field[mLines[i].a] == field[mLines[i].b]) &&
         (field[mLines[i].b] == field[mLines[i].c]) )
    {
      winner = field[mLines[i].a];
      break;
    }
  }

  // Field won?
  if (winner) 
  {
    if (player == winner) return FIELD_WON;
    else return -FIELD_WON;
  }
  // Field drawn
  if (occupied == 9) return FIELD_DRAW;

  winner = -1; // Still running

  // Abort search if we check only for wins
  if (check_won) return 0;

  // Check number of open 2 rows
  int two_rows_player = 0;
  int two_rows_opponent = 0;
  for (i=0;i<9;i++)
  {
    // If field is empty we make a dummy move in the field
    // and check whether the field is then won. If so it
    // was an open row of length 2!
    if (field[i] == 0) 
    {
      int tmp_winner;
      field[i] = player;
      if (calc_value(field,player,tmp_winner,true)>0) two_rows_player++;
      field[i] = opponent;
      if (calc_value(field,opponent,tmp_winner,true)>0) two_rows_opponent++;
      field[i] = 0; // Reset it to 0 (we now here it was zero!)
    }
  }

  // Add a bit of heuristic field values
  int heuristic_player = 0;
  int heuristic_opponent = 0;
  if (field[4] == player) heuristic_player += FIELD_CENTER; 
  if (field[4] == opponent) heuristic_opponent += FIELD_CENTER; 

  if (field[0] == player) heuristic_player += FIELD_CORNER; 
  if (field[0] == opponent) heuristic_opponent += FIELD_CORNER; 
  if (field[2] == player) heuristic_player += FIELD_CORNER; 
  if (field[2] == opponent) heuristic_opponent += FIELD_CORNER; 
  if (field[6] == player) heuristic_player += FIELD_CORNER; 
  if (field[6] == opponent) heuristic_opponent += FIELD_CORNER; 
  if (field[8] == player) heuristic_player += FIELD_CORNER; 
  if (field[8] == opponent) heuristic_opponent += FIELD_CORNER; 


  int value = 0;
  value += two_rows_player * FIELD_ROW_2; // Add rows of len 2
  value -= two_rows_opponent * FIELD_ROW_2; 

  value += heuristic_player;   // Add field heuristic
  value -= heuristic_opponent;

  return value;
}

#ifdef USEKDE
#include "ai.moc"
#endif
