/***************************************************************************
    Border of the fields
                          
    copyright            : (C) 2002 by Martin Heni
    email                : martin@heni-online.de
 ***************************************************************************/

// Qt includes
#include <qcanvas.h>
#include <qpen.h>
#include <qbrush.h>
#include <qpointarray.h>

#include "bordersprite.h"

// =================================================
BorderSprite::BorderSprite(QCanvas *canvas,QSize size, QColor color, QColor backcolor)
            : QCanvasPolygon(canvas)
{

  QPointArray mPoints(4);
  // Create four edge points
  mPoints.setPoint(0,3,3);
  mPoints.setPoint(1,size.width()-3,3);
  mPoints.setPoint(2,size.width()-3,size.height()-3);
  mPoints.setPoint(3,3,size.height()-3);

  mColor = color;
  mBackColor = backcolor;

  // Depth
  setZ(-10);

  // Assign the to the polygon
  setPoints(mPoints);
  // and immediately show the sprite
  show();
}

// Destruct and hide the sprite
BorderSprite::~BorderSprite()
{
  hide();
}


// This is overwritten from the polycon sprite
// and allows to draw the polygon ourselves. We
// use this two draw an 'X' using two lines
void BorderSprite::drawShape(QPainter &p)
{
   // Retrieve the polygon edges
  QPointArray pa=areaPoints();

 QPen pen(mColor,1);
  QBrush brush(mBackColor);

  p.setPen(pen);
  p.setBrush(brush);

  p.drawPolygon(pa);
   QColor lightColor = mColor.light(170);
   QColor darkColor = mColor.dark();
//   QColor lightColor = QColor(QColor(5,5,5));
//   QColor darkColor = QColor(QColor(5,5,5));

   QPen penlight(lightColor,1);
   QPen pendark(darkColor,1);

   p.setPen(penlight);
   p.drawLine(pa.point(0)+QPoint(-1,-1),pa.point(1)+QPoint(2,-1));
   p.drawLine(pa.point(0)+QPoint(-1,-1),pa.point(3)+QPoint(-1,2));

   p.setPen(pendark);
   p.drawLine(pa.point(1)+QPoint(1,0),pa.point(2)+QPoint(1,2));
   p.drawLine(pa.point(2)+QPoint(1,1),pa.point(3)+QPoint(0,1));

   p.setPen(pen);

  return;
}

