/***************************************************************************
                          ktritoc  -  a KDE game dice.cpp
                             -------------------
    begin                : November 2002
    copyright            : (C) 2002 by M. Heni
    email                : ktritoc@heni-online.de
  ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef USEKDE
#include <kdebug.h>
#endif

#include "dice.h"
#include "sprite.h"
#include "geometry.h"


#define DICE_IDLE 0
#define DICE_RUNNING 1
#define DICE_DONE  2

#define DICE_NO_ANIM_FRAMES 5

#define DICE_PROP_12_2  4   // In modified statitics take 1/4 2 and 12's

Dice::Dice()    : QObject(0,"dice")
{
  mStatus = DICE_IDLE;
  mCount = 0;
  mValue = 0;
  mSpriteList.setAutoDelete(true);
  mRandom.setSeed(0);

  mTimer = new QTimer(this);
  mTimer->stop();
  connect(mTimer,SIGNAL(timeout()), this,SLOT(throw_dice()));

  // true: normal dice, false: 2-12:equal probability
  mFairDice=false;  // false: equal distro, true: normal dice
}

Dice::~Dice()
{
  if (mTimer) delete mTimer;
  clear();
}

/** Sets dice back to idle */
void Dice::reset()
{
  if (mTimer) mTimer->stop();
  mStatus = DICE_IDLE;
  mCount = 0;

   // Set Frames
   QCanvasSprite *sprite;
   for ( sprite = mSpriteList.first(); sprite; sprite = mSpriteList.next() )
   {
      sprite->setFrame(0);
   }

}

/** Throw the dice. Returns false if no more turns available */
bool Dice::throwit()
 {
     if (mStatus!=DICE_IDLE && mStatus!=DICE_DONE) return false ;

     if (!throwPossible())
     {
       return false;
     }
     mCount ++ ;
     mStatus = DICE_RUNNING;
     mAnim = DICE_NO_ANIM_FRAMES;
     mTimer->start(TIMER_COUNT,true);
     return true;
}

void Dice::throw_dice()
{

   // Set Frames
   Sprite *sprite;

   // Reset dice
   mValue = 0;
   int rnd=0;

   // kdDebug() << "throw_dice (true=normal dice) fair=" << mFairDice << endl;
   // cout << "throw_dice (true=normal dice) fair=" << mFairDice << endl;

   // Normal dice statistics
   if (mFairDice)
   {
     for ( sprite = mSpriteList.first(); sprite; sprite = mSpriteList.next() )
     {
        rnd = 1+(int)mRandom.getLong(sprite->frameCount()-1);
        // kdDebug() << "D"<<sprite<<"="<<rnd<<endl;
        sprite->setFrame(rnd);
        mValue += rnd;
     }
   }
   // equal distribution
   else
   {
     int spriteno=mSpriteList.count();
     sprite = mSpriteList.first();
     // Draw equally distributed number
     // Sprite count is 7 because of empty frame
     rnd = spriteno+(int)mRandom.getLong(1+(sprite->frameCount()-2)*spriteno) ;
     // Modify statistics for 2 and 12 by redrawing dice in 75% of the cases
     // This reduces the 2 and 12 to 33% of the other numbers drawn!
     //               2                     12
     if ( (rnd == DICE_AMOUNT || rnd == DICE_MAXNUMBER) && mRandom.getLong(DICE_PROP_12_2)>0 )
     {
       rnd = spriteno+(int)mRandom.getLong(1+(sprite->frameCount()-2)*spriteno) ;
     }

     // kdDebug() << "Fair rnd="<<rnd<<endl;
     int tmpcnt=1;
     for ( sprite = mSpriteList.first(); sprite; sprite = mSpriteList.next() )
     {
       // Draw new random number
       int tmprnd = 1+(int)mRandom.getLong(sprite->frameCount()-1);
       // kdDebug() << "Dice " << tmpcnt << "="<<tmprnd<<endl;
       // At least this needs to remain for other dice
       int tmpmindiff = rnd-tmprnd-(spriteno-tmpcnt);
       // At most this can be done by remaining dice
       int tmpmaxdiff = rnd-tmprnd-6*(spriteno-tmpcnt);
       // If no room adjust rnd
       if (tmpmindiff<0) tmprnd+=tmpmindiff;
       if (tmpmaxdiff>0) tmprnd+=tmpmaxdiff;

       // kdDebug() << "tmpmindiff=" << tmpmindiff << " tmpmaxdiff=" << tmpmaxdiff<< " tmprnd=" << tmprnd<<endl;
       tmpcnt++;
       // Adjust remaining rnd
       rnd-=tmprnd;

       // kdDebug() << "D"<<sprite<<"="<<rnd<<endl;
       sprite->setFrame(tmprnd);
       mValue += tmprnd;
     }
   }

  if (mAnim>0) // repeat
  {
     mAnim--;
     mTimer->start(TIMER_COUNT,true);
     return ;
  }
  mStatus = DICE_DONE;
  emit signal_dice_thrown(this);
}
/** Returns dice status */
int Dice::status()
{
  return mStatus;
}
/** Returns dice value or 0 if invalid */
int Dice::value()
{
  if (status() != DICE_DONE)  return 0;
  return mValue;
}

  /** Adds a dice image */
void Dice::AddDice(QCanvas *mCanvas,QString name)
{
     Sprite *sprite = new Sprite(mCanvas,name,1+DICE_SIZE);
     if (!sprite) return;
     int i = mSpriteList.count();
     sprite->move(mOffset.x() + DICE_X*i,mOffset.y());
     sprite->setFrame(0);
     mSpriteList.append(sprite);
}

/** Clear out sprites */
void Dice::clear()
{
  mSpriteList.clear();
}
/** Checks whether a given pointis inside the dice graphics */
bool Dice::inside(QPoint point)
{
  QPoint dice = QPoint(point.x()-mOffset.x(),point.y()-mOffset.y());
  int count = mSpriteList.count();
  if (dice.x()>-DICE_X/2 && dice.x()<DICE_X*count-DICE_X/2 &&
       dice.y()>-DICE_Y/2 && dice.y()<DICE_Y/2)
   {
    return true;
   }
   return false;
}
/** Set graphic offset */
void Dice::setOffset(QPoint point)
{
  mOffset = point;
}
/** Sets the dice to a given status */
void Dice::setFinished()
{
  mStatus = DICE_DONE;
  mCount = MAX_COUNT; // no more thorws
}
/** Returns fals if no more dice can be thrown */
bool Dice::throwPossible()
{
  if (mCount>=MAX_COUNT) return false;
  return true;
}

/** Sets the dice mode (0:normal, 1:equal prob)*/
void Dice::setDiceMode(int mode)
{
  if (mode == 0) mFairDice = true;
  else mFairDice = false;
}
/** Gets the dice mode */
int Dice::getDiceMode()
{
  if (mFairDice) return 0;
  else return 1;
}

#ifdef USEKDE
#include "dice.moc"
#endif
