/***************************************************************************
                          ktritoc  -  a KDE game dice.h
                             -------------------
    begin                : November 2002
    copyright            : (C) 2002 by M. Heni
    email                : ktritoc@heni-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DICE_H
#define DICE_H


#include <qobject.h>
#include <qcanvas.h>
#include <qpoint.h>
#include <qtimer.h>
#ifdef USEKDE
#include <qptrlist.h>
#else
#include <qlist.h>
#define QPtrList QList
#endif

#include "mrandom.h"



class Sprite;


/**
  *@author M. Heni
  */

class Dice : public QObject
{
  Q_OBJECT

public: 
  Dice();
  ~Dice();

  /** Adds a dice image */
  void AddDice(QCanvas *canvas,QString name);
  /** Sets dice back to idle */
  void reset();
  /** Throw the dice. Returns false if no more turns available */
  bool throwit();
  /** Returns dice value or 0 if invalid */
  int value();
  /** Clear out sprites */
  void clear();

  /** Checks whether a given pointis inside the dice graphics */
  bool inside(QPoint point);
  /** Set graphic offset */
  void setOffset(QPoint point);
  /** Sets the dice to a given status */
  void setFinished();
  /** Returns fals if no more dice can be thrown */
  bool throwPossible();

  /** Sets the dice mode (0:normal, 1:equal prob)*/
  void setDiceMode(int mode);
  /** Gets the dice mode */
  int getDiceMode();

protected:
  /** Returns dice status */
  int status();


public slots:

  void throw_dice();


private:
  QPtrList<Sprite> mSpriteList;
  MRandom mRandom;
  int mCount;
  int mStatus;
  int mValue;
  int mAnim;
  /** Normal dice or equal distribution */
  bool mFairDice;
  QPoint mOffset;
  QTimer *mTimer;

signals: // Signals
  /** Is emmited when the dice are thrown */
  void signal_dice_thrown(Dice *dice);
};

#endif
