/***************************************************************************
                          gameboard  -  a KDE game gameboard.cpp
                             -------------------
    begin                : November 2002
    copyright            : (C) 2002 by M. Heni
    email                : ktritoc@heni-online.de
  ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <math.h>

// include files for Qt
#include <qdir.h>
#include <qwidget.h>

// include files for KDE
#ifdef USEKDE
#include <kdebug.h>
#include <stdlib.h>
#endif

// application specific includes
#include "gameboard.h"
#include "sprite.h"
#include "numbersprite.h"
#include "bordersprite.h"
#include "movesprite.h"
#include "geometry.h"



GameBoard::GameBoard(int no) : QObject(0,"gameboard")
{
  //                       9
  mPieceSprites.resize(FIELD_AMOUNT);
  mPieceSprites.setAutoDelete(true);
  mNumberSprites.resize(FIELD_AMOUNT);
  mNumberSprites.setAutoDelete(true);
  mEmptySprites.resize(FIELD_AMOUNT);
  mEmptySprites.setAutoDelete(true);
  mMoveSprites.resize(FIELD_AMOUNT);
  mMoveSprites.setAutoDelete(true);
  mBorderSprite = 0;
  mMarkerSprite = 0;

  mFields.resize(FIELD_AMOUNT);
  //                       12
  mDiceLookup.resize(1+DICE_MAXNUMBER);
  mNumber.resize(FIELD_AMOUNT);

  mRandom.setSeed(0);

  mId = no;
  reset();
}

GameBoard::~GameBoard()
{
  clear();
}

// Resets board element and calculates numbering
// depening on given index 0
void GameBoard::reset()
{
  int i;
  mOwner = 0;
  // Reset owner of fields
  for (i=0;i<FIELD_AMOUNT;i++)
  {
    mFields[i]=0;
    // FAST QUIT: mFields[i]= (i==0)?0:1;
    if (mPieceSprites.at(i)) mPieceSprites.at(i)->setFrame(mFields[i]);
  }

  // Mapping dice no -> field
  //               12
  for (i=0;i<=DICE_MAXNUMBER;i++)
  {
    mDiceLookup[i]=-1;
  }

  // Mapping field-> dice no
  for (i=0;i<FIELD_AMOUNT;i++)
  {
    //               Min=3          Start +3        No=12 - 3 = 9
    mNumber[i]=(DICE_AMOUNT+1) + (5+i+mId)% (DICE_MAXNUMBER-3);
    // kdDebug() << "Set field "<<no << "["<<i << "] to "<<number[i]<<endl;
    mDiceLookup[mNumber[i]]=i;
  }

  if (mMarkerSprite) mMarkerSprite->hide();
}
/** Sets the board offset */
void GameBoard::setOffset(QPoint offset)
{
  mOffset = offset;
}
/** Loads sprites from disk an adds them */
void GameBoard::AddBoard(QCanvas *canvas, QString piece, QString empty,QString marker)
 {
     QPoint destpos;
     QPoint offset;
     for (int i=0;i<FIELD_AMOUNT;i++) //TODO
     {
       //            3
       int x = i % SIZE_FIELD_X;
       int y = i / SIZE_FIELD_X;
       QPoint fieldoffset = QPoint(FIELD_OFFSET_X+x*FIELD_X,FIELD_OFFSET_Y+y*FIELD_Y);

       // The piece
       Sprite *sprite = new Sprite(canvas,piece,3);
       #ifdef USEKDE
       if (!sprite) {kdError() << "Cannot load sprite " << piece << endl;exit(1);}
       #else
       if (!sprite) return ;
       #endif
       mPieceSprites.insert(i,sprite);
       destpos = mOffset + fieldoffset;
       sprite->move(destpos.x(),destpos.y());
       sprite->setFrame(mFields[i]);
       sprite->setZ(30);

       // The numbers
       int value = mNumber.at(i);
       QString str = QString("%1").arg(value);
       NumberSprite *number = new NumberSprite(canvas,str,FONTSIZE);
       #ifdef USEKDE
       if (!number) {kdError() << "Cannot load sprite " << str << endl;exit(1);}
       #else
       if (!number) return ;
       #endif
       mNumberSprites.insert(i,number);
       QPoint hotspot = QPoint((sprite->width())/2,(sprite->height())/2);
       offset = QPoint(NUMBER_OFFSET_X,NUMBER_OFFSET_Y);
       destpos = mOffset+fieldoffset-hotspot+offset;
       number->move(destpos.x(),destpos.y());
       number->setZ(20);

        // The holes
       Sprite *hole = new Sprite(canvas,empty);
       #ifdef USEKDE
       if (!hole) {kdError() << "Cannot load sprite " << empty << endl;exit(1);}
       #else
       if (!hole) return ;
       #endif
       mEmptySprites.insert(i,hole);
       destpos = mOffset+fieldoffset;
       hole->move(destpos.x(),destpos.y());
       hole->setZ(10);
    }

  // Border
  QSize bordersize = QSize(BOARD_X,BOARD_Y);
  BorderSprite *border = new BorderSprite(canvas, bordersize, COL_BORDER,COL_BOARD);
  #ifdef USEKDE
  if (!border) {kdError() << "Cannot create " << "border" << endl;exit(1);}
  #else
  if (!border) return ;
  #endif
  destpos = mOffset;
  border->move(destpos.x(),destpos.y());
  mBorderSprite = border;


  // The marker
  mMarkerSprite = new Sprite(canvas,marker,2);
  #ifdef USEKDE
  if (!mMarkerSprite) {kdError() << "Cannot load sprite " << marker << endl;exit(1);}
  #else
  if (!mMarkerSprite) return ;
  #endif
  mMarkerSprite->hide();
  mMarkerSprite->setZ(40);

  // The moving pieces
  for (int j=0;j<FIELD_AMOUNT;j++)
  {
    MoveSprite *movesprite = new MoveSprite(canvas,piece,3);
    #ifdef USEKDE
    if (!movesprite) {kdError() << "Cannot load sprite " << piece << endl;exit(1);}
    #else
    if (!movesprite) return ;
    #endif
    mMoveSprites.insert(j,movesprite);
    movesprite->setZ(70);
  }


}
/** Checks whether point is inside the board and sets x,y coordinates then */
bool GameBoard::inside(QPoint point, int &x, int &y)
{
  x = -1;
  y = -1;
// Get the board
  point -= mOffset;
  if (point.x()>=0 && point.x()<BOARD_X &&
       point.y()>=0 && point.y()<BOARD_Y)
  {
   //kdDebug() << "x=" << point.x() << " y=" << point.y() << endl;
   QPoint fieldoffset = QPoint(FIELD_OFFSET_X+MOUSE_OFFSET_X,
                               FIELD_OFFSET_Y+MOUSE_OFFSET_Y);
   point -= fieldoffset;
   x = point.x()/FIELD_X;
   y = point.y()/FIELD_Y;
   //       3                  3
   if (x<SIZE_FIELD_X && y<SIZE_FIELD_Y)   return true;
   else return false;
  }
  return false;
}

/** Sets the given (x,y) field to a value 0:none, 1:yellow, 2:red
  * Returns the real field number */
int GameBoard::setField(int x, int y, int value)
{
  int i= x+SIZE_FIELD_X*y;
  mFields[i] = value;
  mPieceSprites.at(i)->setFrame(mFields[i]);
  return i;
}

/** Deletes all sprites */
void GameBoard::clear()
{
  int i;
  //             9
  for (i=0;i<FIELD_AMOUNT;i++)
  {
     mPieceSprites.remove(i);
     mEmptySprites.remove(i);
     mNumberSprites.remove(i);
  }
  //             9
  for (i=0;i<FIELD_AMOUNT;i++)
  {
     mMoveSprites.remove(i);
  }
  if (mBorderSprite)
  {
     delete mBorderSprite;
     mBorderSprite = 0;
  }

  if (mMarkerSprite)
  {
     delete mMarkerSprite;
     mMarkerSprite = 0;
  }

}

/** Returns contents of given field (1=yellow,2=red) */
int GameBoard::getField(int x, int y)
 {
     //           3
     int i= x+SIZE_FIELD_X*y;
     return mFields[i];
}

/** Returns the dice value of the given field */
int GameBoard::getFieldValue(int x, int y)
{
     //           3
     int i= x+SIZE_FIELD_X*y;
    return mNumber[i]; // Dice value fo field i
}

/** Returns the value of a field given a field number, see getField */
int GameBoard::getFieldFromNumber(int no)
{
  return mFields[no];
}

/** Returns the value of a field given a dice number, see getField */
int GameBoard::getFieldFromDice(int dice)
{
  int no = mDiceLookup[dice];
  if (no<0) return -1;
  return mFields[no];
}
/** Checks whether the current board is taken over by one player */
bool GameBoard::checkTakenOver()
{
  if (mOwner) return false;  // Already won

  int i;
  //              3
  for (i=0;i<SIZE_FIELD_X;i++)
  {
  // Check rows
     int row = SIZE_FIELD_X*i;
     if (mFields[row]      == mFields[row+1] &&
          mFields[row+1] == mFields[row+2] &&
          mFields[row] !=0)
     {
        winBoard(mFields[row]); // Player wins board
        return true;
     }

     // Check cols
     if (mFields[i]      == mFields[3+i] &&
          mFields[3+i] == mFields[6+i] &&
          mFields[i] !=0)
     {
        winBoard(mFields[i]); // Player wins board
        return true;
     }
  } // next i

  // Check diagonal 1
     if (mFields[0]   == mFields[4] &&
          mFields[4]   == mFields[8] &&
          mFields[0] !=0)
     {
        winBoard(mFields[0]); // Player wins board
        return true;
     }
  // Check diagonal 2
     if (mFields[2]   == mFields[4] &&
          mFields[4]   == mFields[6] &&
          mFields[2] !=0)
     {
        winBoard(mFields[2]); // Player wins board
        return true;
     }

     return false;


}
/** Player col wins the board - all pieces turned */
void GameBoard::winBoard(int col)
{
  int x;
  int y;
  int cnt = 0;

  for (y=0;y<SIZE_FIELD_Y;y++)
  {
    for (x=0;x<SIZE_FIELD_X;x++)
    {
      int i=x+SIZE_FIELD_X*y;
      if (mFields[i] != col)
      {
         if (mFields[i] != 0) // opponet
         {
           // start movment
           mMoveSprites.at(cnt)->setFrame(mFields[i]);
           mMoveSprites.at(cnt)->setX(mPieceSprites.at(i)->x());
           mMoveSprites.at(cnt)->setY(mPieceSprites.at(i)->y());
           double angle = mRandom.getDouble() * 100.0 +40.0;
           double rad_angle = angle/180.0*M_PI;
           //                                   70.0          60.0
           double speed = mRandom.getDouble() * BOARD_V_RND + BOARD_V_OFFSET;
           speed *= VELOCITY_MULTI;
           mMoveSprites.at(cnt)->setVelocity(cos(rad_angle)*speed,-1.0*sin(rad_angle)*speed);
           mMoveSprites.at(cnt)->setMovement(1,5.0); // Gravity
           cnt++;
         }
         setField(x,y,col);
      }
    }// next x
  }// next y
  mOwner = col;


}
/** Puts a sprite marker on the given number (-1: removes it)*/
void GameBoard::markNumber(int i,int frame)
{
   //              9
   if (i<0 || i>FIELD_AMOUNT )
   {
      mMarkerSprite->hide();
      return ;
   }
   int x = i % SIZE_FIELD_X;
   int y = i / SIZE_FIELD_X;
   // kdDebug() << "Moving dice "<<no<<" as marker "<<i<<" to " << x << "," << y << endl;

   QPoint fieldoffset = QPoint(FIELD_OFFSET_X+x*FIELD_X,FIELD_OFFSET_Y+y*FIELD_Y);
   QPoint destpos = mOffset+fieldoffset;
   mMarkerSprite->setFrame(frame);
   mMarkerSprite->move(destpos.x(),destpos.y());
   mMarkerSprite->show();

}

/** Checks whether the board has an owner, i.e. is occupied */
int GameBoard::getOwner(){ return mOwner; }

/** Returns the number of free pieces of this board */
int GameBoard::NumberOfFreePieces()
{
  int count = 0;
  //                 9
  for (int i=0;i<FIELD_AMOUNT;i++)
  {
    if (mFields[i] == 0) count++;
  }
  return count;
}

/** Gets number of field from dice value */
bool GameBoard::getNumberFromDice(int dice, int &no)
{
   no = mDiceLookup[dice];
   if (no<0) return false;
   return true;
}

/** Gets coordinates of field from dice value */
bool GameBoard::getXYFromDice(int dice, int &x, int &y)
{
  int no = mDiceLookup[dice];
  if (no<0) return false;

  x=no % SIZE_FIELD_X;
  y=no / SIZE_FIELD_X;
  return true;
}
#ifdef USEKDE
#include "gameboard.moc"
#endif
