/***************************************************************************
                          gameboard  -  a KDE game - gameboard.h
                             -------------------
    begin                : November 2002
    copyright            : (C) 2002 by M. Heni
    email                : ktritoc@heni-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GAMEBOARD_H
#define GAMEBOARD_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qpoint.h>
#include <qcanvas.h>
#ifdef USEKDE
#include <qptrvector.h>
#include <qmemarray.h>
#else
#include <qvector.h>
#include <qarray.h>
#define QPtrVector QVector
#define QMemArray QArray
#endif

#include "mrandom.h"


class Sprite;
class NumberSprite;
class BorderSprite;
class MoveSprite;

class GameBoard : public QObject
{
  Q_OBJECT

  public:
  GameBoard(int id);
  ~GameBoard();
    void reset();
  /** Sets the board offset */
  void setOffset(QPoint offset);
  /** Loads sprites from disk an adds them */
  void AddBoard(QCanvas *canvas, QString piece, QString empty,QString marker);
  /** Checks whether point is inside the board and sets x,y coordinates then */
  bool inside(QPoint point, int &x, int &y);
  /** Sets the given (x,y) field to a value 0:none, 1:yellow, 2:red */
  int setField(int x, int y, int value);
  /** Deletes all sprites */
  void clear();
  /** Returns contents of given field (1=yellow,2=red) */
  int getField(int x, int y);
  /** Returns the dice value of the given field */
  int getFieldValue(int x, int y);

  /** Returns the value of a field given a dice number */
  int getFieldFromDice(int dice);
  /** Returns the value of a field given a field number */
  int getFieldFromNumber(int number);
  /** Checks whether the current board is taken over by one player */
  bool checkTakenOver();
  /** Player col wins the board - all pieces turned */
  void winBoard(int col);
  /** Puts a sprite marker on the given number */
  void markNumber(int no,int frame=0);
  /** Checks whether the board has an owner, i.e. is occupied */
  int getOwner();
  /** Returns the number of free pieces of this board */
  int NumberOfFreePieces();
  /** Gets coordinates of field from dice value */
  bool getXYFromDice(int dice, int &x, int &y);
  /** Gets number of field from dice value */
  bool getNumberFromDice(int dice, int &no);
  private:


    Sprite *mMarkerSprite;
    BorderSprite *mBorderSprite;
    QMemArray<int> mFields;     // owner of field 0,1,2
    QMemArray<int> mDiceLookup;     // dice X goes to field Y
    QMemArray<int> mNumber;    // field X is dice Y
    int mOwner;                  // who ons the board 0,1,2
    int mId;                         // Id or number of board
    QPoint mOffset;            // offset of board
    QPtrVector<Sprite> mPieceSprites;
    QPtrVector<NumberSprite> mNumberSprites;
    QPtrVector<Sprite> mEmptySprites;
    QPtrVector<MoveSprite> mMoveSprites;
    MRandom mRandom;
};



#endif //GAMEBOARD_H
