/***************************************************************************
                          ktritoc  -  a KDE game gameinfo.cpp
                             -------------------
    begin                : November 2002
    copyright            : (C) 2002 by M. Heni
    email                : ktritoc@heni-online.de
  ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 #ifdef USEKDE
 #include <kdebug.h>
 #include <stdlib.h>
 #include <klocale.h>
 #else
 #define i18n tr
 #endif
 #include <qtimer.h>

#include "gameinfo.h"
#include "sprite.h"
#include "numbersprite.h"
#include "geometry.h"



GameInfo::GameInfo() : QObject(0,"gameinfo")
{
  mInfoText = 0;
  mInfoSprite = 0;
  mTimer = new QTimer(this);
  mTimer->stop();
  connect(mTimer,SIGNAL(timeout()),
                this,SLOT(hideInfo()));

}

GameInfo::~GameInfo()
{
  if (mTimer) delete mTimer;
}

void GameInfo::setOffset(QPoint offset)
{
  mOffset = offset;
}

void GameInfo::clear()
{
  if (mInfoSprite)
  {
    delete mInfoSprite;
    mInfoSprite = 0;
  }
  if (mInfoText)
  {
    delete mInfoText;
    mInfoText = 0;
  }

}


/** Adds info sprite */
void GameInfo::AddInfo(QCanvas *canvas,QString info)
{
  QPoint dest;
  QPoint field;

  Sprite *sprite =   new Sprite(canvas,info,GAMEINFO_AMOUNT);
  #ifdef USEKDE
  if (!sprite) {kdError() << "Cannot load sprite "<<info<<endl;exit(1);}
  #else
  if (!sprite) return ;
  #endif
  mInfoSprite = sprite;
  field = QPoint(INFO_OFFSET_SPRITE_X,INFO_OFFSET_SPRITE_Y);
  dest = mOffset + field;
  sprite->move(dest.x() ,dest.y());
  sprite->setZ(50);
  sprite->hide();

  // The text
  QString str = i18n("Welcome");
  NumberSprite *number = new NumberSprite(canvas,str,INFO_FONTSIZE);
  #ifdef USEKDE
  if (!number) {kdError() << "Cannot load sprite " << str << endl;exit(1);}
  #else
  if (!number) return ;
  #endif
  mInfoText = number;
  field = QPoint(INFO_OFFSET_TEXT_X,INFO_OFFSET_TEXT_Y);
  dest = mOffset + field;
  number->move(dest.x(),dest.y());
  number->setZ(45);
  number->hide();
}

/** Shows info icon and text */
void GameInfo::setInfo(int no,QString text, int timeout)
{
  mInfoSprite->hide();
  mInfoText->hide();

  // Sprite
  if (no>=0)
  {
    mInfoSprite->setFrame(no);
    mInfoSprite->show();
  }

  // Text
  if (!text.isNull())
  {
    mInfoText->setText(text);
    mInfoText->show();
  }
  else

  if (timeout>0)
  {
    mTimer->stop();
    mTimer->start(timeout,true);
  }
}

/*
  * hides all sprites after timeout
  */
void GameInfo::hideInfo()
{
  if (mInfoText) mInfoText->hide();
  if (mInfoSprite) mInfoSprite->hide();
}

/** Resets info */
void GameInfo::reset()
{
  hideInfo();
}

#ifdef USEKDE
#include "gameinfo.moc"
#endif
