/***************************************************************************
                          ktritoc  -  a KDE game gameintro.cpp
                             -------------------
    begin                : November 2002
    copyright            : (C) 2002 by M. Heni
    email                : ktritoc@heni-online.de
  ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//#include <stream.h>
#include <math.h>

#include <qpixmap.h>
#include <qimage.h>

#ifdef USEKDE
#include <kdebug.h>
#include <stdlib.h>
#endif

#include "gameintro.h"
#include "movesprite.h"
#include "geometry.h"

#define TIME_WAIT1  6000     /* ms to wait to reinstall intro */
#define TIME_WAIT2  12000    /* ms to wait until remove of intro */

#define GRAVITY 0
#define LINEAR  1

GameIntro::GameIntro()    : QObject(0,"gameintro")
{
  mSpriteList.setAutoDelete(true);
  mRandom.setSeed(0);

  mTimer = new QTimer(this);
  mTimer->stop();
  connect(mTimer,SIGNAL(timeout()), this,SLOT(start_remove()));

  mCanvas= 0;


}

GameIntro::~GameIntro()
{
  // Delete sprites
  mSpriteList.clear();
  if (mTimer) delete mTimer;
}

void GameIntro::LoadIntro(QCanvas *canvas,QString piece,QString intro)
{
  mCanvas = canvas;
  // Delete sprites
  mSpriteList.clear();


  QPixmap *pixmap=new QPixmap(intro);
  if (!pixmap || pixmap->isNull())
  {
    #ifdef USEKDE
    kdError() << "WARNING: No intropicture defined."<<endl;
    #endif
    return ;
  }
  
  // Access x,y points
  QImage img=pixmap->convertToImage();  
  if (img.width()<1 || img.height()<1) 
  {
    #ifdef USEKDE
    kdError() << "WARNING: Intropicture size ZERO."<<endl;
    #endif
    return ;
  }

  // Scaling between image and canvas
  double x_factor = 8.0; // canvas->width()/img.width();
  double y_factor = 8.0; //canvas->height()/img.height();

  int frame;
  int spritecnt = 0;

  for (int y=0;y<img.height();y++)
  {
    for (int x=0;x<img.width();x++)
    {
      QColor col(img.pixel(x,y));
      if (col.red()>127 && col.green()>127) frame=1;
      else if (col.red()>127 ) frame=2;
      else continue;

      int r0=mRandom.getLong(4);
      int x0;
      int y0;

      if (r0==1) // TR
      {
        x0=mRandom.getLong(40)-20;
        y0=mRandom.getLong(40)-20;
        x0+=canvas->width();
      }
      else if (r0==2) // BL
      {
        x0=mRandom.getLong(40)-20;
        y0=mRandom.getLong(40)-20;
        y0+=canvas->height();
      }
      else if (r0==3) // BR
      {
        x0=mRandom.getLong(40)-20;
        y0=mRandom.getLong(40)-20;
        x0+=canvas->width();
        y0+=canvas->height();
      }
      else
      {
        x0=mRandom.getLong(40)-20;
        y0=mRandom.getLong(40)-20;
      }
       
      int x1 = (int)(x*x_factor);
      int y1 = (int)(y*y_factor);

      double dist_x = x1-x0;
      double dist_y = y1-y0;

      double vx,vy;

      vx = dist_x/INTRO_LINEAR_DURATION_FACTOR;   
      vy = dist_y/INTRO_LINEAR_DURATION_FACTOR;   
      
      //double vx = 2.0;
      //double vy = 2.0;
      //if (x1<x0) vx = -vx;
      //if (y1<y0) vy = -vy;

      /*
      kdDebug() << "Sprite ("<<x0<<","<<y0<<") =>"
                             <<y1<<","<<y1<<") "
                << " v= (" << vx<<","<<vy<<")" << endl;
      */

      MoveSprite *sprite =   new MoveSprite(canvas,piece,3);
      #ifdef USEKDE
      if (!sprite) {kdError() << "Cannot load sprite "<<piece<<endl;exit(1);}
      #else
      if (!sprite) return ;
      #endif
      mSpriteList.append(sprite);
      sprite->setVelocity(vx*VELOCITY_MULTI,vy*VELOCITY_MULTI);
      sprite->setFrame(frame);
      sprite->move(x0,y0);
      sprite->setDestination(x1,y1);
      sprite->setZ(30+spritecnt);
      sprite->setMovement(2,0.0); // Linear
      spritecnt++;
    } // x
  } // y
}

void GameIntro::start_remove()
{
  MoveSprite *sprite;
  if (mMode == GRAVITY)
  {
    for ( sprite = mSpriteList.first(); sprite; sprite = mSpriteList.next() )
    {
      // Store old location
      sprite->setDestination(sprite->x(),sprite->y());
      double angle = mRandom.getDouble() * 100.0 +40.0;
      double rad_angle = angle/180.0*M_PI;
      //                                    130.0         100.0
      double speed = mRandom.getDouble() * INTRO_V_RND+ INTRO_V_OFFSET;
      speed *= VELOCITY_MULTI;
      sprite->setVelocity(cos(rad_angle)*speed,-1.0*sin(rad_angle)*speed);
      sprite->setMovement(1,5.0); // Gravity
      mTimer->start(TIME_WAIT1,true);
    }// next for
    mMode = LINEAR;
  }// end GRAVITY  
  else if (mMode == LINEAR)
  {
    for ( sprite = mSpriteList.first(); sprite; sprite = mSpriteList.next() )
    {
      double x1,y1;
      sprite->getDestination(x1,y1);

      double x0,y0;
      y0 = -50; // mCanvas->height()+50;
      sprite->setY(y0);

      x0 = sprite->x();
      y0 = sprite->y();

      double dist_x = x1-x0;
      double dist_y = y1-y0;

      double vx,vy;

      vx = dist_x/INTRO_LINEAR_DURATION_FACTOR;   
      vy = dist_y/INTRO_LINEAR_DURATION_FACTOR;   

      sprite->setVelocity(vx*VELOCITY_MULTI,vy*VELOCITY_MULTI);
      sprite->setMovement(2,0.0); // Linear

    }// next for
    mMode = GRAVITY;
    mTimer->start(TIME_WAIT2,true);
  }

}

void GameIntro::StartIntro()
{
  mTimer->start(TIME_WAIT2,true);
  mMode = GRAVITY;
}

#ifdef USEKDE
#include "gameintro.moc"
#endif
