/***************************************************************************
                          ktritoc  -  a KDE game ktritocdoc.cpp
                             -------------------
    begin                : November 2002
    copyright            : (C) 2002 by M. Heni
    email                : ktritoc@heni-online.de
  ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qwidget.h>

// include files for KDE
#ifdef USEKDE
#include <klocale.h>
#include <kdebug.h>
#include <kconfig.h>
#else
#include "kconfig.h"
#define i18n tr
#endif

// application specific includes
#include "ktritocdoc.h"
#include "ktritoc.h"
#include "ktritocview.h"
#include "dice.h"
#include "gameboard.h"
#include "gameinfo.h"
#include "gameover.h"
#include "score.h"
#include "geometry.h"
#include "ai.h"


// Who plays whom
#define HUMAN_PLAYER 0
#define COMPUTER_PLAYER 1

// Mode of the game
#define GAMESTATUS_INTRO 0
#define GAMESTATUS_RUNNING 1
#define GAMESTATUS_GAMEOVER 2

#define LEVEL_RANDOM_FILL 7

KtritocDoc::KtritocDoc(QWidget *parent, const char *name) : QObject(parent, name)
{

  mRandom.setSeed(0);

  mLoadCurrentPlayer = -1;
  loadBoardConfig=0;
  mCurrentGames = 0;
  mDiceChanges = 0;
  mView = 0;
  mAi = 0;
  mGameStatus = GAMESTATUS_INTRO;
  mLoadGameStatus = mGameStatus;
  mGameInfo = new GameInfo();
  mGameInfo->setOffset(QPoint(INFO_OFFSET_X,INFO_OFFSET_Y));
  mGameOver = new GameOver();
  mGameOver->setOffset(QPoint(GAMEOVER_OFFSET_X,GAMEOVER_OFFSET_Y));
  mDice = new Dice();
  // Init dice
  mDice->setOffset(QPoint(DICE_OFFSET_X,DICE_OFFSET_Y));
  // Connect slot
  connect(mDice,SIGNAL(signal_dice_thrown(Dice *)),
                this, SLOT(slot_dice_thrown(Dice *)) );

  mScoreValue.resize(2); // Value of score
  mPlayedBy.resize(2);     // Who plays whom
  mScore.resize(2); // Scores
  mScore.setAutoDelete(true);
  for (int i=0;i<2;i++)
  {
    mScore.insert(i,new Score(i));
    QPoint point = QPoint(SCORE_OFFSET_X+i*SCORE_X,SCORE_OFFSET_Y);
    mScore.at(i)->setOffset(point);
    mScoreValue[i] = 0;
    mPlayedBy[i] = HUMAN_PLAYER;
  }

  mBoard.resize(BOARD_AMOUNT); 
  for (int i=0;i<BOARD_AMOUNT;i++)
  {
    mBoard.insert(i,new GameBoard(i)); 
    int x = i % SIZE_BOARD_X; 
    int y = i / SIZE_BOARD_X; 
    QPoint point = QPoint(BOARD_OFFSET_X+x*BOARD_X,BOARD_OFFSET_Y+y*BOARD_Y);

    mBoard.at(i)->setOffset(point);
   }
   mBoard.setAutoDelete(true);

   // new computer Ai
   mAi = new Ai(this);



  // Default settings
  mLevel = 0; // Default level 
  mMaxGames = 1; // 1 games to play
  mStartPlayer = 1;  // start player
  mPlayedBy[0] = HUMAN_PLAYER;
  mPlayedBy[1] = COMPUTER_PLAYER;
}

KtritocDoc::~KtritocDoc()
{
   if (mDice) delete mDice;
   if (mGameInfo) delete mGameInfo;
   if (mGameOver) delete mGameOver;
   if (mAi) delete mAi;

}

void KtritocDoc::setView(KtritocView *view)
{
  mView = view;
}

/** Resets all game variables */
void KtritocDoc::resetGame()
{
  int i;
  mCurrentPlayer = -1;
  mDiceChanges = 0;

  // Stop Ai
  mAi->stop();

  for (i=0;i<BOARD_AMOUNT;i++)
  {
    mBoard.at(i)->reset();
  }

  for (i=0;i<2;i++)
  {
    mScore.at(i)->reset();
    mScore.at(i)->setScore(mScoreValue[i]); 
  }

  getGameInfo()->reset();
  getDice()->reset();
  getGameOver()->StopGameOver();

}

/** Prefill the board if a level is given
  *
  */
void KtritocDoc::presetLevel()
{
  // Level 0 is normal
  if (mLevel == 0) return ;
  
  // Fill color
  int col;
  if (mLevel>0) col=2;
  else col=1;

  // Strength..always>0
  int mode = mLevel;
  if (mode<0) mode=-mode;

  int i;
  // Fill 5 random pieces
  if (mode > 0)
  {
    for (i=0;i<LEVEL_RANDOM_FILL;i++)
    {
      int b = mRandom.getLong(BOARD_AMOUNT);
      int x = mRandom.getLong(SIZE_FIELD_X);
      int y = mRandom.getLong(SIZE_FIELD_Y);
      mBoard.at(b)->setField(x,y,col);
    }
  }

  // Fill middle pieces
  if (mode == 2)
  {
    mBoard.at(0)->setField(1,1,col);
    mBoard.at(2)->setField(1,1,col);
    mBoard.at(4)->setField(1,1,col);
    mBoard.at(6)->setField(1,1,col);
    mBoard.at(8)->setField(1,1,col);
  }

  // Make sure whether that taken over is checked
  // But give NO score points
  for (i=0;i<BOARD_AMOUNT;i++)
  {
    getBoard(i)->checkTakenOver();
  }



}

/** Starts new game */
void KtritocDoc::newGame()
{
 mGameStatus = GAMESTATUS_RUNNING;

 // Reset game
 resetGame();

 // Create board
 mView->drawBoard();

 // Prefill board according to level
 // or load from config file if it is the first time and
 // the game was RUNNING and a proper current player is set
 if (loadBoardConfig && mLoadGameStatus == GAMESTATUS_RUNNING &&
     mLoadCurrentPlayer>0)
 {
   loadBoard(loadBoardConfig);
   loadBoardConfig = 0; // Don't do it again
   // Switch current player as nextPlayer will switch again
   mCurrentPlayer = 3-mLoadCurrentPlayer;
 }
 else
 {
   presetLevel();
 }


 // Increase game number
 mCurrentGames++;

 // Set startplayer
 nextPlayer();
}

/** Returns board of the given number */
GameBoard* KtritocDoc::getBoard(int no)
 {
  return mBoard.at(no);
}

/** Returns dice */
Dice* KtritocDoc::getDice()
 {
  return mDice;
}
/** Returns current player (1=yellow, 2=red) */
int KtritocDoc::getCurrentPlayer()
{
  return mCurrentPlayer;
}

/** Switches the players */
bool KtritocDoc::nextPlayer()
{
  if (mCurrentPlayer == 1) mCurrentPlayer = 2;
  else if (mCurrentPlayer == 2) mCurrentPlayer = 1;
  else mCurrentPlayer = mStartPlayer; // new game
  getScore(mCurrentPlayer-1)->setPlayerStatus(true);
  getScore(1-(mCurrentPlayer-1))->setPlayerStatus(false);
  getGameInfo()->setInfo(GAMEINFO_THROWDICE,i18n("throw dice"),GAMEINFO_TIMEOUT);

  // Start Ai?
  if (mPlayedBy[mCurrentPlayer-1] == COMPUTER_PLAYER)    mAi->throwDice();

  return true;
}

/** Checks whether the queried number is still free in any board */
bool KtritocDoc::numberStillFree(int no)
{
   for (int i=0;i<BOARD_AMOUNT;i++) // loop boards
   {
   int col = mBoard.at(i)->getFieldFromDice(no);
   // kdDebug() << "Board:"<<i<<" col on dice("<<no<<")="<<col<<endl;
   if (col==0) return true; // found
   }
   // kdDebug() << "Number " << no << " NOT FREE"<<endl;
  // Nothing found
  return false;
}

/** Called when the dice is thrown */
void KtritocDoc::slot_dice_thrown(Dice *dice)
{
   int i;
   int value =  getDice()->value();
  
   // Remove all old marker
   for (i=0;i<BOARD_AMOUNT;i++)
   {
     getBoard(i)->markNumber(-1);
   }
   
   #ifdef USEKDE
   kdDebug() << "slot_dice_thrown value=" << value << endl;
   #endif
   if (numberStillFree(value))
   {
     dice->setFinished(); // we are done now!

     // Show make move icon dependent on current player
     if (mCurrentPlayer == 1)
       getGameInfo()->setInfo(GAMEINFO_MAKEMOVE1,i18n("make move"),GAMEINFO_TIMEOUT);
     else
       getGameInfo()->setInfo(GAMEINFO_MAKEMOVE2,i18n("make move"),GAMEINFO_TIMEOUT);

     // Mark all boards free numbers
     for (i=0;i<BOARD_AMOUNT;i++)
     {
       int number=-1; //negativ number: hide marker 
       if (!getBoard(i)->getNumberFromDice(value,number)) number = -1;
       if (getBoard(i)->getFieldFromDice(value)!=0) number = -1;
       getBoard(i)->markNumber(number,0);
     } 

     // Start Ai?
     if (mPlayedBy[mCurrentPlayer-1] == COMPUTER_PLAYER)    mAi->makeMove(mCurrentPlayer);

    // kdDebug() << "DICE ("<<value<<")FINISHED set"<<endl;
   }
   else if (canRemovePiece(value))
   {
      //              2 resp 12
      if ( (value == DICE_AMOUNT    && mCurrentPlayer == 1) ||
           (value == DICE_MAXNUMBER && mCurrentPlayer == 2) )
        getGameInfo()->setInfo(GAMEINFO_REMOVE_YELLOW,i18n("remove piece"),GAMEINFO_TIMEOUT);
     if ( (value == DICE_AMOUNT    && mCurrentPlayer == 2) ||
          (value == DICE_MAXNUMBER && mCurrentPlayer == 1 ) )
        getGameInfo()->setInfo(GAMEINFO_REMOVE_RED,i18n("remove piece"),GAMEINFO_TIMEOUT);
     dice->setFinished(); // we are done now!

    // Start Ai?
    if (mPlayedBy[mCurrentPlayer-1] == COMPUTER_PLAYER)  mAi->makeMove(mCurrentPlayer);
   }
   else if (!dice->throwPossible())
   {
      mDiceChanges++;
      // kdDebug() << "No more throws and number engaged - switch player"<<mDiceChanges<<endl;
      checkGameOver();
      if (mGameStatus != GAMESTATUS_GAMEOVER)
      {
        getGameInfo()->setInfo(GAMEINFO_NOMOVE,i18n("no move"),GAMEINFO_TIMEOUT);
        // Reset dice and swap player
        getDice()->reset();
        nextPlayer();
      }
   }
   else
   {
     // kdDebug() << "Dice can be thrown again"<<endl;
     getGameInfo()->setInfo(GAMEINFO_THROWDICE,i18n("throw dice"),GAMEINFO_TIMEOUT);
    // Start Ai?
    if (mPlayedBy[mCurrentPlayer-1] == COMPUTER_PLAYER)    mAi->throwDice();
   }
}

/** Finishes the move, removes stuff etc. Returns the game status (0:cont, 1: game over) */
int KtritocDoc::finishMove(int board,int number)
{
  // Check occupation of board
  if (getBoard(board)->checkTakenOver())
  {
    getScore(mCurrentPlayer-1)->startDollar();
    mScoreValue[mCurrentPlayer-1]+=1; // SCORE
    mScore.at(mCurrentPlayer-1)->setScore(mScoreValue[mCurrentPlayer-1]); 
  }
  // Reset dice and swap player
  getDice()->reset();

  // Hide all marker
  for (int i=0;i<BOARD_AMOUNT;i++) getBoard(i)->markNumber(-1);

  // Mark move
  if (board>=0 && number>=0) getBoard(board)->markNumber(number,1);

  // Checks game over situation
  checkGameOver();

  // Reset dice changes
  mDiceChanges = 0;
  // kdDebug() << "Setting dice changes to 0" << endl;

  // Next player if still running
  if (mGameStatus  == GAMESTATUS_RUNNING)
  {
    nextPlayer();
  }

  return 0;
}

/** Gets pointer to score element */
Score * KtritocDoc::getScore(int no)
{
  return mScore.at(no);
}

/** Checks whether a piece can be removed (value=2 or 12) */
bool KtritocDoc::canRemovePiece(int value)
{
  int  what;
  //               2
  if (value == DICE_AMOUNT) what = mCurrentPlayer;
  //                   12
  else if (value == DICE_MAXNUMBER) what = 3-mCurrentPlayer;
  else return false; // wrong dice

   for (int i=0;i<BOARD_AMOUNT;i++) // loop boards
   {
      if (mBoard.at(i)->getOwner()!=0) continue; // omit full boards
      // Loop all pieces
      //             3           12
      for (int j=1+DICE_AMOUNT;j<DICE_MAXNUMBER;j++) 
      {
         int col = mBoard.at(i)->getFieldFromDice(j);
         if (col == what)
         {
            // kdDebug() << "can remove Board:"<<i<<"  dice("<<j<<")="<<col<<endl;
            return true;
         }
     } // next j
   } // next i
   // kdDebug() << "CANNOT REMOVE"<<endl;
  // Nothing found
  return false;

}
/** Gets the game info object */
GameInfo * KtritocDoc::getGameInfo(){ return mGameInfo; }

/** Gets the game over object */
GameOver * KtritocDoc::getGameOver(){ return mGameOver; }

/** Checks the status of the game (0:intro,1:RUNNING, 2:gameover) */
bool KtritocDoc::isRunning()
{ 
  return mGameStatus == GAMESTATUS_RUNNING;
}

/** Analyse whether the game is over. Set the status then and
emit a signal */
void KtritocDoc::checkGameOver()
{
  int i;
  // kdDebug() << "Check game over" << endl;

  // Check for blocked dice
  bool throw_stop = false;
  if (mDiceChanges>=MAX_DICE_CHANGES)
  {
    throw_stop = true;
    #ifdef USEKDE
    kdDebug() << "Too many dice changes..end game" << endl;
    #endif
  }


  // Check for a free board
  bool is_free = false;
  int won_player[3]; // Fields won
  won_player[0] = 0;
  won_player[1] = 0;
  won_player[2] = 0;
  for (i=0;i<BOARD_AMOUNT;i++)
  {
    if (getBoard(i)->getOwner()<3) won_player[getBoard(i)->getOwner()]++;
    if (getBoard(i)->getOwner()==0 && getBoard(i)->NumberOfFreePieces()>0)
    {
      is_free = true;
    }
  }// next i

  #ifdef USEKDE
  kdDebug() << "is_free=" << is_free << " Empty=" << won_player[0]<< endl;
  kdDebug() << "Player1 =" << won_player[1] << " player2="<< won_player[2]<<endl;
  #endif

  int winner = 0;
  // Check for win situation
  for (i=0;i<3;i++)
  {
  // Check rows
     int row = 3*i;
     if (getBoard(row)->getOwner()   == getBoard(row+1)->getOwner() &&
         getBoard(row+1)->getOwner() == getBoard(row+2)->getOwner() &&
         getBoard(row)->getOwner() !=0)
     {
        winner = getBoard(row)->getOwner(); 
     }

     // Check cols
     if (getBoard(i)->getOwner()   == getBoard(3+i)->getOwner() &&
         getBoard(3+i)->getOwner() == getBoard(6+i)->getOwner() &&
         getBoard(i)->getOwner() !=0)
     {
        winner = getBoard(i)->getOwner(); 
     }
  } // next i

  // Check diagonal 1
     if (getBoard(0)->getOwner()  == getBoard(4)->getOwner() &&
         getBoard(4)->getOwner()  == getBoard(8)->getOwner() &&
         getBoard(0)->getOwner() !=0)
     {
        winner = getBoard(0)->getOwner(); 
     }
  // Check diagonal 2
     if (getBoard(2)->getOwner()  == getBoard(4)->getOwner() &&
         getBoard(4)->getOwner()  == getBoard(6)->getOwner() &&
         getBoard(2)->getOwner() !=0)
     {
        winner = getBoard(2)->getOwner(); 
     }

     #ifdef USEKDE
     kdDebug() << "winner=" << winner << endl;
     #endif

     if (winner!=0 || !is_free || throw_stop)
     {
       // Check scoring
       // winning player: +5
       // more boards than opponent: +1  DISABLED
       // opponent has no boards: +1     DISABLED
       int delta[2];
       delta[0]=0;
       delta[1]=0;
       // if (won_player[1]>won_player[2]) delta[0]++;
       // if (won_player[1]>0 && won_player[2]==0) delta[0]++;
       if (winner==1) delta[0]+=5;

       // if (won_player[2]>won_player[1]) delta[1]++;
       // if (won_player[2]>0 && won_player[1]==0) delta[1]++;
       if (winner==2) delta[1]+=5;

       // Add score
       mScoreValue[0]+=delta[0];
       mScoreValue[1]+=delta[1];

       // Run sprite
       getGameOver()->StartGameOver();
       // Change status
       mGameStatus = GAMESTATUS_GAMEOVER;

       for (i=0;i<2;i++)
       {
        mScore.at(i)->setScore(mScoreValue[i]); 
        // TODO mScore.at(i)->setDelta(delta[i]);
       }

       // Emit signal
       emit signalGameOver(winner);
     } 
}

/**
 * Checks whether current player is a human player and game 
 * is running
 */
bool KtritocDoc::currentMouseInput()
{
 if (mGameStatus != GAMESTATUS_RUNNING) return false;
 if (mPlayedBy[mCurrentPlayer-1] != HUMAN_PLAYER) return false;
 return true;
}

/** Set Startplayer */
void KtritocDoc::setStartplayer(int no)
{
  mStartPlayer = no;
}
/** Get Startplayer */
int KtritocDoc::getStartplayer() { return mStartPlayer; }

/** Set Level */
void KtritocDoc::setLevel(int no)
{
  mLevel = no;
}
/** Get Level */
int KtritocDoc::getLevel() {return mLevel; }

/** Set Ai mode (player, false:human, true:computer) */
void KtritocDoc::setAiMode(int player, bool compi)
{
  int old = mPlayedBy[player-1];
  mPlayedBy[player-1] = compi?COMPUTER_PLAYER:HUMAN_PLAYER;

  // If mode is really changed and it is the current player
  // we have to start or stop the AI
  if (old != mPlayedBy[player-1] && player == mCurrentPlayer &&
      mGameStatus == GAMESTATUS_RUNNING)
  {
    #ifdef USEKDE
    kdDebug() << "Changing Ai MODE" << endl;
    #endif
    if (mPlayedBy[player-1] == COMPUTER_PLAYER)
    {
      getDice()->reset();
      mAi->throwDice();
    }
    else mAi->stop();
  }
  
}

/** Get Ai mode for player */
bool KtritocDoc::getAiMode(int player)
{
  if (mPlayedBy[player-1] == COMPUTER_PLAYER)  return true;
  return false;
}


/** Set number of games */
void KtritocDoc::setMaxGames(int no)
{
  mMaxGames = no;
}
/** Get number of games */
int KtritocDoc::getMaxGames() {return mMaxGames; }

/** Set dice mode (0:normal,1:equal)  */
void KtritocDoc::setDiceMode(int no)
{
  mDice->setDiceMode(no);
}
/** Get dice mode (0:normal,1:equal)  */
int KtritocDoc::getDiceMode() {return mDice->getDiceMode(); }

bool KtritocDoc::gameMaxReached()
{
  // TODO: Unless highscore is used we return always false
  return false;

  if (mCurrentGames>=mMaxGames)
  {
    mCurrentGames = 0;
    return true;
  }
  return false;
}

void KtritocDoc::resetScores()
{
    mScore.at(0)->setScore(0); 
    mScore.at(1)->setScore(0); 
    mScoreValue[0] = 0;
    mScoreValue[1] = 0;
}


// ------------------- LOAD/SAVE -----------------------
/** Write document data to config file
 */
void KtritocDoc::readProperties(KConfig *config)
{
  config->setGroup("GeneralData");
  // Game status
  mLoadGameStatus=config->readNumEntry("gamestatus",mGameStatus);
  // Startplayer
  mStartPlayer=config->readNumEntry("startplayer",mStartPlayer);
  // Currentplayer
  mLoadCurrentPlayer=config->readNumEntry("currentplayer",mCurrentPlayer);
  // Level
  mLevel=config->readNumEntry("level",mLevel);
  // maxgames
  mMaxGames=config->readNumEntry("maxgames",mMaxGames);
  // currentgames
  mCurrentGames=config->readNumEntry("currentgames",mCurrentGames);
  // dicemode
  int dicemode=config->readNumEntry("dicemode",getDiceMode());
  setDiceMode(dicemode);
  // Aimode
  int ai1=config->readNumEntry("aimode1",(int)getAiMode(1));
  setAiMode(1,ai1==0?false:true);
  int ai2=config->readNumEntry("aimode2",(int)getAiMode(2));
  setAiMode(2,ai2==0?false:true);

  // Score
  mScoreValue[0]=config->readNumEntry("score1",mScoreValue[0]);
  mScoreValue[1]=config->readNumEntry("score2",mScoreValue[1]);

  // Remember config to load from
  loadBoardConfig=config;

}

/** Load the game board from a config file
 */ 
void KtritocDoc::loadBoard(KConfig *config)
{
  config->setGroup("Board");
  for (int i=0;i<BOARD_AMOUNT;i++)
  {
    QString name = QString("Board%1").arg(i);
    QValueList<int> field=config->readIntListEntry(name);
    for (int y=0;y<SIZE_FIELD_Y;y++)
    {
      for (int x=0;x<SIZE_FIELD_X;x++)
      {
        int col = field.first();
        field.remove(field.begin()); // Remove first item
        // field.pop_front(); // Remove first item
        getBoard(i)->setField(x,y,col);
      }
    }
    getBoard(i)->checkTakenOver();
  }// next i
}

/** Read document data from config file
 */
void KtritocDoc::saveProperties(KConfig *config)
{
  config->setGroup("GeneralData");
  // Game status
  config->writeEntry("gamestatus",mGameStatus);
  // Startplayer
  config->writeEntry("startplayer",mStartPlayer);
  // Currentplayer
  config->writeEntry("currentplayer",mCurrentPlayer);
  // Level
  config->writeEntry("level",mLevel);
  // maxgames
  config->writeEntry("maxgames",mMaxGames);
  // currentgames
  config->writeEntry("currentgames",mCurrentGames);
  // dicemode
  config->writeEntry("dicemode",getDiceMode());
  // Aimode
  config->writeEntry("aimode1",(int)getAiMode(1));
  config->writeEntry("aimode2",(int)getAiMode(2));
  // Score
  config->writeEntry("score1",mScoreValue[0]);
  config->writeEntry("score2",mScoreValue[1]);

  config->setGroup("Board");
  for (int i=0;i<BOARD_AMOUNT;i++)
  {
    QString name = QString("Board%1").arg(i);
    QValueList<int> field;
    for (int y=0;y<SIZE_FIELD_Y;y++)
    {
      for (int x=0;x<SIZE_FIELD_X;x++)
      {
        int no = getBoard(i)->getField(x,y);
        field.append(no);
      }
    }
    #ifdef USEKDE
    config->writeEntry(name,field);
    #else
    config->writeIntListEntry(name,field);
    #endif
  }// next i

  // Sync it
  #ifdef USEKDE
  config->sync();
  #endif
}


#ifdef USEKDE
#include "ktritocdoc.moc"
#endif
