/***************************************************************************
                          ktritoc  -  a KDE game ktritocdoc.h
                             -------------------
    begin                : November 2002
    copyright            : (C) 2002 by M. Heni
    email                : ktritoc@heni-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KTRITOCDOC_H
#define KTRITOCDOC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for QT
#include <qobject.h>
#include <qstring.h>
#ifdef USEKDE
#include <qptrvector.h>
#include <qmemarray.h>
#else
#include <qvector.h>
#include <qarray.h>
#define QPtrVector QVector
#define QMemArray QArray
#endif


#include "mrandom.h"

class GameBoard;
class Dice;
class Score;
class GameInfo;
class GameOver;
class Ai;
class KConfig;


// forward declaration of the Ktritoc classes
class KtritocView;



/**	KtritocDoc provides a document object for a document-view model.
  *
  * The KtritocDoc class provides a document object that can be used in conjunction with the classes KtritocApp and KtritocView
  * to create a document-view model for standard KDE applications based on KApplication and KMainWindow. Thereby, the document object
  * is created by the KtritocApp instance and contains the document structure with the according methods for manipulation of the document
  * data by KtritocView objects. Also, KtritocDoc contains the methods for serialization of the document data from and to files.
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. 	
  * @version KDevelop version 1.2 code generation
  */
class KtritocDoc : public QObject
{
  Q_OBJECT
  public:
    /** Constructor for the fileclass of the application */
    KtritocDoc(QWidget *parent, const char *name=0);
    /** Destructor for the fileclass of the application */
    ~KtritocDoc();

    /** adds a view to the document which represents the document contents. Usually this is your main view. */
    void setView(KtritocView *view);
  /** Resets all game variables */
  void resetGame();
  /** Starts new game */
  void newGame();
  /** Returns board of the given number */
  GameBoard *getBoard(int no);
  /** Returns dice object */
  Dice *getDice();
  /** Switches the players */
  bool nextPlayer();
  /** Returns current player (1=yellow, 2=red) */
  int getCurrentPlayer();
  /** Checks whether the queried number is still free in any board */
  bool numberStillFree(int no);
  /** Finishes the move, removes stuff etc. Returns the game status (0:cont, 1: game over) */
  int finishMove(int board,int number);
  /** Gets pointer to score element */
  Score * getScore(int no);
  /** Gets the game info object */
  GameInfo * getGameInfo();
  /** Gets the game over object */
  GameOver * getGameOver();
  /** Checks if the game is still running */
  bool isRunning();

  /** Checks whether current player has mouse input (is human and running) */
  bool currentMouseInput();

  /** ACCESS Functions of the settings menu */
  /** Set Startplayer */
  void setStartplayer(int no);
  /** Get Startplayer */
  int getStartplayer();

  /** Set Level */
  void setLevel(int no);
  /** Get Level */
  int getLevel();

  /** Set Ai mode (player, false:human, true:computer) */
  void setAiMode(int player, bool compi);
  /** Get Ai mode for player */
  bool getAiMode(int player);



  /** Set number of games */
  void setMaxGames(int no);
  /** Get number of games */
  int getMaxGames();

  /** Set dice mode (0:normal,1:equal)  */
  void setDiceMode(int no);
  /** Get dice mode */
  int getDiceMode();

  /** Check whether max game number is reached */
  bool gameMaxReached();
  /** Reset the scores */
  void resetScores();

  /** LOAD/SAVE */
  void readProperties(KConfig *cfg);
  void saveProperties(KConfig *cfg);


  public slots:

signals: // Signals
   /** Game over signal */
   void signalGameOver(int winner);
 	
public:	
    KtritocView *mView;

protected slots: // Protected slots
  /** Called when the dice is thrown */
  void slot_dice_thrown(Dice *dice);

protected: // Protected methods
  /** Checks whether a piece can be removed (value=2 or 12) */
  bool canRemovePiece(int value);
  /** Analyse whether the game is over. Set the status then and emit a signal */
  void checkGameOver();
  /** Fill the board according to level */
  void presetLevel();
  /** Loads board from config object */
  void loadBoard(KConfig *config);


private:
  QMemArray<int> mScoreValue;  // Score of player 0,1
  QMemArray<int> mPlayedBy;    // Player i played by what?
  QPtrVector<GameBoard> mBoard;
  Ai  *mAi;
  Dice *mDice;
  GameOver *mGameOver;
  GameInfo *mGameInfo;
  QPtrVector<Score> mScore;
  int   mCurrentPlayer;             // Whose turn?
  int   mLoadCurrentPlayer;            // Whose turn (saved)?
  int   mStartPlayer;        // Who starts
  /** Game status: 1:running, 0:intro, 2:gameover */
  int mGameStatus;
  int mLoadGameStatus; // Gamestatus as load from file
  /** How often did the dice change */
  int mDiceChanges;
  int mMaxGames; // Max number of games 
  int mCurrentGames; // current game number
  int mLevel;  // Current level (2=normal default)
  MRandom mRandom;
  KConfig *loadBoardConfig;
};

#endif // KTRITOCDOC_H
