/***************************************************************************
                          ktritoc  -  a KDE game ktritocview.cpp
                             -------------------
    begin                : November 2002
    copyright            : (C) 2002 by M. Heni
    email                : ktritoc@heni-online.de
  ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


// include files for KDE
#ifdef USEKDE
#include <kdebug.h>
#endif

// application specific includes
#include "ktritocview.h"
#include "ktritocdoc.h"
#include "ktritoc.h"
#include "numbersprite.h"
#include "bordersprite.h"
#include "sprite.h"
#include "dice.h"
#include "gameboard.h"
#include "score.h"
#include "gameinfo.h"
#include "gameintro.h"
#include "gameover.h"
#include "geometry.h"


#define DEBUG_GAME 0  // allow cheats

#define VALUE_REMOVE_OWN  2
#define VALUE_REMOVE_OTHER 12


KtritocView::KtritocView(QWidget *parent, const char *name): QCanvasView(0,parent, name)

{
  // Store the parent
  mParent = parent;

  // Reset status
  mStatus = 0;


  // We do not need scrolling so switch it off
  setVScrollBarMode(AlwaysOff);
  setHScrollBarMode(AlwaysOff);

  // Set window background (not needed)
  setBackgroundColor(COL_BACK);

  // Create a new canvas for this view
  mCanvas=new QCanvas(parent);
  // Fit it in the parent window
  mCanvas->resize(parent->width(),parent->height());
  // Remove flicker by double buffering (probably not
  // needed here but we keep it for demonstration)
  mCanvas->setDoubleBuffering(true);
  // Choose a nice background color
  mCanvas->setBackgroundColor(COL_BACK);
  // Update/advance
  mCanvas->setAdvancePeriod(25);
  // assign the canvas tot he view
  setCanvas(mCanvas);

  // Set size and position
  move(0,0);
  adjustSize();

  // and prepare the sprite list
  // mSpriteList.setAutoDelete(true);

  // Init the random generator
  mRandom.setSeed(0);



  // Start timer
  // mTimer = new QTimer(this);
  // connect( mTimer, SIGNAL(timeout()), this, SLOT(timerDone()) );

  mIntro = 0;

  // Allow for initialisation
  QTimer::singleShot(0,this,SLOT(drawIntro()));
  // drawIntro();

}

KtritocView::~KtritocView()
{
  if (mIntro) delete mIntro;
  if (mCanvas) delete mCanvas;
}

void KtritocView::resizeEvent(QResizeEvent *e)
{
  if (mCanvas) mCanvas->resize(e->size().width(),e->size().height());
}

KtritocDoc *KtritocView::getDocument() const
{
  KtritocApp *theApp=(KtritocApp *) parentWidget();

  return theApp->getDocument();
}

/** Draws the full board in its current shape (normally the start board) */
void KtritocView::drawBoard()
{

   // clear();  // clear remains

  // First time init
  if (mStatus==0)
  {
    drawPieces();
    drawDice();
    drawScore();
    drawInfo();
    drawGameOver();
    mStatus = 1;

    // Delete intro
    if (mIntro) 
    {
      delete mIntro;
      mIntro = 0;
    }
  }


}
/** Remove all board content */
void KtritocView::clear()
{
  //mSpriteList.clear();
//  getDocument()->getDice()->clear();
//  for (int i=0;i<9;i++)
//  {
//    getDocument()->getBoard(i)->clear();
//  }
//  getDocument()->getGameInfo()->clear();
}


/** Draw all board pieces */
void KtritocView::drawPieces()
 {
   //              9
  for (int i=0;i<BOARD_AMOUNT;i++) 
  {
     getDocument()->getBoard(i)->AddBoard(mCanvas,
           mGrafix+QString("piece%1.png"),
           mGrafix+QString("empty.png"),
           mGrafix+QString("marker%1.png"));
    getDocument()->getBoard(i)->reset();
  }
}

/** Draws the dice */
void KtritocView::drawDice()
{
  //                 2
  for (int i=0;i<DICE_AMOUNT;i++)
  {
     getDocument()->getDice()->AddDice(mCanvas,
                           mGrafix+QString("purple%1.png"));
  }
  getDocument()->getDice()->reset();
}

/** Draws the game info */
void KtritocView::drawInfo()
{
    getDocument()->getGameInfo()->AddInfo(mCanvas,
                           mGrafix+QString("info%1.png"));
}

/** Draws the game intro */
void KtritocView::drawIntro()
{
    mIntro = new GameIntro();
    mIntro->LoadIntro(mCanvas, 
                      mGrafix+QString("piece%1.png"),
                      mGrafix+QString("intro.png"));
    mIntro->StartIntro();
}

 /** Draws the game over */
void KtritocView::drawGameOver()
{
    getDocument()->getGameOver()->AddSprite(mCanvas,
                      mGrafix+QString("game_over%1.png"));
}

// mouse click event
void KtritocView::contentsMousePressEvent( QMouseEvent *mouse )
{
  if (mouse->button()!=LeftButton) return ;
  KtritocDoc *doc = getDocument();

  // Checks running and human player. Otherwise no mouse input
  if (!doc->currentMouseInput()) return ;


  QPoint point;
  point=mouse->pos();

  if (doc->getDice()->inside(point))
  {
    if (!doc->getDice()->throwit())
    {
      #ifdef USEKDE
      kdDebug() << "Can't throw right now"<<endl;
      #endif
    }
     return ;
  }// end inside dice

  // Dice thrown
  int dicevalue = doc->getDice()->value();
   if (dicevalue<=0)
   {
     #ifdef USEKDE
     kdDebug() << "DICE not thrown properly" << endl;
     #endif
     return ;
   }


   // Check field
   for (int i=0;i<BOARD_AMOUNT;i++)
   {
     int x;
     int y;
     if (doc->getBoard(i)->inside(point,x,y))
     {
       // kdDebug() << "Board click("<<i<<"):" << x << "," << y << endl;
       if (dicevalue == VALUE_REMOVE_OWN &&
           doc->getBoard(i)->getField(x,y) ==doc->getCurrentPlayer() &&
           doc->getBoard(i)->getOwner() == 0)
       {
         // kdDebug() << "remove own ("<<i<<"):" << x << "," << y << endl;
         int no = doc->getBoard(i)->setField(x,y,0);
         doc->finishMove(i,no );
       }
       if (dicevalue == VALUE_REMOVE_OTHER &&
           doc->getBoard(i)->getField(x,y) == 3-doc->getCurrentPlayer() &&
           doc->getBoard(i)->getOwner() == 0)
       {
         // kdDebug() << "remove other ("<<i<<"):" << x << "," << y << endl;
         int no = doc->getBoard(i)->setField(x,y,0);
         doc->finishMove(i,no);
       }
       else if (doc->getBoard(i)->getFieldValue(x,y) != dicevalue && !DEBUG_GAME)
       {
         #ifdef USEKDE
         kdDebug() << "not a dice number" << endl;
         kdDebug() << "field value is"<< doc->getBoard(i)->getFieldValue(x,y)<< endl;
         #endif
       }
       else if ( doc->getBoard(i)->getField(x,y)==0)
       {
         // Set move
         int no = doc->getBoard(i)->setField(x,y,doc->getCurrentPlayer());
         doc->finishMove(i,no);
         // Next player
       }
       else
       {
         #ifdef USEKDE
         kdDebug() << "OCCUPIED" << endl;
         #endif
       }
     }
   } // next for


}


/** Draws score widget */
void KtritocView::drawScore()
{
  getDocument()->getScore(0)->AddScore(mCanvas,
                                mGrafix+QString("yellow%1.png"),
                                mGrafix+QString("score.png"),
                                mGrafix+QString("piece1.png"),
                                mGrafix+QString("dollar.png")
                                               );
  getDocument()->getScore(1)->AddScore(mCanvas,
                                mGrafix+QString("red%1.png"),
                                mGrafix+QString("score.png"),
                                mGrafix+QString("piece2.png"),
                                mGrafix+QString("dollar.png")
                                               );
}

#ifdef USEKDE
#include "ktritocview.moc"
#endif
