/***************************************************************************
                          ktritoc  -  a KDE game movesprite.cpp
                             -------------------
    begin                : November 2002
    copyright            : (C) 2002 by M. Heni
    email                : ktritoc@heni-online.de
  ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef USEKDE
#include <kdebug.h>
#endif
#include <math.h>
#include "movesprite.h"

#define MOVEMENT_NONE 0
#define MOVEMENT_GRAVITY 1
#define MOVEMENT_LINEAR 2
#define MOVEMENT_BROWNIAN_X 3

#define TIME_STEP  0.025  // Advance time step:should correspond to canvas

#ifdef USEKDE
#define GRAVITY_FACTOR 130.0
#else
#define GRAVITY_FACTOR 260.0
#endif

MoveSprite::MoveSprite(QCanvas *Canvas, QString filename, int frames) :
                   Sprite(Canvas,filename,frames)
{
  mMovement = MOVEMENT_NONE;  // Default
  mTime = 0.0;
  mMaxtime = 5.0;
  mAnimDelay = 0;
  mAnimCount = 0;
  mAnimType = 0; // Cycle
  hide();
  mRandom.setSeed(0);
  mBrownDir = 1;
}

MoveSprite::~MoveSprite(){
}

// Provides the sprite animation and movement
void MoveSprite::advance(int phase)
{
  if (phase == 0) // Collision
  {
    // Call parents function to do the move for us
    QCanvasSprite::advance(phase);
  }
  else // movement and animation
  {
    if (mAnimDelay>0)
    {
      mAnimCount++;
      if (mAnimCount>=mAnimDelay)
      {
        mAnimCount = 0;
        // Cyclic animation
        int nf = frame();
        nf = (nf + 1 + frameCount()) % frameCount();
        setFrame(nf);
      }
    }

    // Time over .. hide sprite
    if ((mTime > mMaxtime && mMaxtime>0.0) && mMovement != MOVEMENT_NONE)
    {
       mMovement = MOVEMENT_NONE;
       // kdDebug() << "Stop animation " << endl;
       hide();
    }
    switch(mMovement)
    {
      case MOVEMENT_GRAVITY:  gravity();
      break;
      case MOVEMENT_LINEAR:  linear();
      break;
      case MOVEMENT_BROWNIAN_X:  brownianX();
      break;
    }
  }// end else phase 1
}

// Type = 0: cycle, 1: oscillate
void MoveSprite::setAnimation(int delay=1, int type)
{
  mAnimDelay = delay;
  mAnimCount = 0;
  mAnimType = type;
  setFrame(0); // reset frame
}

/** Set destination point  if applicaple */
void MoveSprite::setDestination(double x, double y)
{
  mDestX = x;
  mDestY = y;
}

void MoveSprite::getDestination(double &x, double &y)
{
  x = mDestX;
  y = mDestY;
}

/** Chooses the movement strategy */
void MoveSprite::setMovement(int no, double maxtime)
{
  // MOVEMENT_GRAVITY, MOVEMENT_LINEAR,...
  mMovement = no;
  mMaxtime = maxtime;
  mTime = 0.0;
  mX = x();     // double
  mY = y();
  setAnimated(true);
  show();
}
/** Performs gravity animation
   * x and y velocity serve as start point */
void MoveSprite::gravity()
{
  double vx = xVelocity();
  double vy = yVelocity();
  double a = GRAVITY_FACTOR; // 130.0
  setX(mX +vx * mTime);
  setY(mY  + vy* mTime + 0.5 * a * mTime * mTime);
   mTime += TIME_STEP;
}

/** Linear movment until destination is reached
  *  */
void MoveSprite::linear()
{
  mTime += TIME_STEP;
 if (xVelocity()>0.0)
 {
   if (x()+xVelocity()>mDestX) {setXVelocity(0.0);setX(mDestX);}
 }
 else if (xVelocity()<0.0)
 {
   if (x()+xVelocity()<mDestX) {setXVelocity(0.0);setX(mDestX);}
 }
 if (yVelocity()>0.0)
 {
   if (y()+yVelocity()>mDestY) {setYVelocity(0.0);setY(mDestY);}
 }
 else if (yVelocity()<0.0)
 {
   if (y()+yVelocity()<mDestY) {setYVelocity(0.0);setY(mDestY);}
 }
 QCanvasSprite::advance(1);
}

/** Linear brownian movment until destination is reached
  * Brownian movement on X direction
  * Otherwise linear
  *  */
void MoveSprite::brownianX()
{
  mTime += TIME_STEP;
  // Change direction ? 
  if (mRandom.getDouble()<0.03) mBrownDir = -mBrownDir;

  // Make move
  double dx = xVelocity() * mBrownDir;

  double away = fabs( (x()-mDestX)/15.0/(dx==0.0?1.0:dx) );
  if (away<1.0) away=1.0;

  // double oldx=x();
  double ddx = dx * 1.0/away;
  setX(x()+ddx);
  //kdDebug() << "BRow  " <<mBrownDir<< " dx=" << dx<< " away=" << away<< " x="<<x()<< " destx="<<mDestX<< "="<<ddx<< "oldx="<<oldx<<endl;

  if (yVelocity()>0.0)
  {
    if (y()+yVelocity()>mDestY) {setYVelocity(0.0);setY(mDestY);}
    else setY(y()+yVelocity());
  }
  else if (yVelocity()<0.0)
  {
    if (y()+yVelocity()<mDestY) {setYVelocity(0.0);setY(mDestY);}
    else setY(y()+yVelocity());
  }
  //QCanvasSprite::advance(1);
}
