/***************************************************************************
                          ktritoc  -  a KDE game score.cpp
                             -------------------
    begin                : November 2002
    copyright            : (C) 2002 by M. Heni
    email                : ktritoc@heni-online.de
  ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <math.h>

#ifdef USEKDE
#include <kdebug.h>
#include <stdlib.h>
#endif

#include "score.h"
#include "sprite.h"
#include "movesprite.h"
#include "geometry.h"

#define SCORE_DIGITS   4

Score::Score(int id) : QObject(0,"score")
{
  mScore = 0;
  mId = id;
  mBackgroundSprite = 0;
  mPlayerSprite = 0;
  mDollarSprite = 0;
  mNumberSprites.resize( SCORE_DIGITS);
  mNumberSprites.setAutoDelete(true);
  mRandom.setSeed(0);

}
Score::~Score(){
      clear();
}
/** Reset score */
void Score::reset()
{
  mScore = 0;
  if (mPlayerSprite) mPlayerSprite->hide();
  if (mDollarSprite) mDollarSprite->hide();
}
/** Sets the score */
void Score::setScore(int value){
   mScore=value;

   // kdDebug() << "Score to "<<mScore<<endl;
   for (int i=0;i<SCORE_DIGITS;i++)
   {
     Sprite *sprite = mNumberSprites.at(i);
     if (sprite)   sprite->setFrame(getDigit(mScore,i+1));
   }
}
/** Returns the score */
int Score::getScore(){
  return mScore;
}
/** Loads sprites for the score */
void Score::AddScore(QCanvas *canvas,QString number,QString background,QString player,QString dollar)
{
    QPoint dest;

     Sprite *back =   new Sprite(canvas,background);
     #ifdef USEKDE
     if (!back) {kdError() << "Cannot load sprite "<<background<<endl;exit(1);}
     #else
     if (!back) return ;
     #endif
     mBackgroundSprite = back;
     back->move(mOffset.x() ,mOffset.y());
     back->setZ(10);

     Sprite *playersprite =   new Sprite(canvas,player);
     #ifdef USEKDE
     if (!playersprite) {kdError() << "Cannot load sprite "<<player<<endl;exit(1);}
     #else
     if (!playersprite) return ;
     #endif
     mPlayerSprite = playersprite;
     QPoint field = QPoint(DIGIT_PLAYER_OFFSET_X,DIGIT_PLAYER_OFFSET_Y);
     dest = field+mOffset;
     playersprite->move(dest.x() ,dest.y());
     playersprite->setZ(30);
     playersprite->hide();

     for (int i=0;i<SCORE_DIGITS;i++)
     {
       Sprite *sprite = new Sprite(canvas,number,10);
       #ifdef USEKDE
       if (!sprite) {kdError() << "Cannot load sprite "<<number<<endl;exit(1);}
       #else
       if (!sprite) return ;
       #endif
       mNumberSprites.insert(i,sprite);
       sprite->setZ(20);
       sprite->setFrame(getDigit(mScore,i+1));
       QPoint field = QPoint(DIGIT_OFFSET_X+i*DIGIT_X,0);
       dest = field+mOffset;
       sprite->move(dest.x(),dest.y());
     }

     MoveSprite *dollarsprite =   new MoveSprite(canvas,dollar);
     #ifdef USEKDE
     if (!dollarsprite) {kdError() << "Cannot load sprite "<<dollar<<endl;exit(1);}
     #else
     if (!dollarsprite) return ;
     #endif
     mDollarSprite = dollarsprite;
     field = QPoint(DOLLAR_OFFSET_X,DOLLAR_OFFSET_Y);
     dest = field+mOffset;
     dollarsprite->move(dest.x() ,dest.y());
     dollarsprite->setZ(60);
     dollarsprite->hide();
}

/** Delets sprites */
void Score::clear(){
  if (mPlayerSprite)
  {
    delete mPlayerSprite;
    mPlayerSprite=0;
  }
  if (mDollarSprite)
  {
    delete mDollarSprite;
    mDollarSprite=0;
  }
  if (mBackgroundSprite)
  {
    delete mBackgroundSprite;
    mBackgroundSprite=0;
  }


  for (int i=0;i<SCORE_DIGITS;i++)
  {
    mNumberSprites.remove(i);
  }
}

/** Starts the flying dollar sign */
void Score::startDollar()
{
  if (!mDollarSprite) return ;
  double xDx = 0.5+0.5*mRandom.getDouble();
  double xDir = mId>0?(-1.0):(1.0);
  QPoint field = QPoint(DOLLAR_OFFSET_X,DOLLAR_OFFSET_Y);
  QPoint source = field+mOffset;
  QPoint dest = source+QPoint(xDx*xDir*DOLLAR_OFFSET_DX,DOLLAR_OFFSET_DY);
  mDollarSprite->move(source.x() ,source.y());
  mDollarSprite->setVelocity(DOLLAR_VX*xDir*VELOCITY_MULTI,DOLLAR_VY*VELOCITY_MULTI);
  mDollarSprite->setDestination(dest.x(),dest.y());
  mDollarSprite->setMovement(2,3.0);
}

/** Sets the position */
void Score::setOffset(QPoint offset){
  mOffset = offset;
}
/** Gets the pos'th digit of the value */
int Score::getDigit(int value,int pos){

  #ifdef USEKDE
  if (pos<1) {kdError()<<"Wrong digit position 0"<<endl;exit(1);}
  #else
  if (pos<1) return 0;
  #endif
  pos = 1+SCORE_DIGITS-pos;
  int n1 = (int)pow(10,pos);
  int n2 = n1/10;

  int res = (value % n1)/n2;
  // kdDebug() <<"digit("<<value<<","<<pos<<")="<<res<<endl;
  return res % 10;
}
/** Activates or deactivates player active/moving */
void Score::setPlayerStatus(bool status){
  if (status && mPlayerSprite) mPlayerSprite->show();
  else if (mPlayerSprite) mPlayerSprite->hide();
}

#ifdef USEKDE
#include "score.moc"
#endif
