/***************************************************************************
    Sprite for pixmap sprites
                          
    copyright            : (C) 2002 by Martin Heni
    email                : ktritoc@heni-online.de
 ***************************************************************************/

// Qt includes
#include <qcanvas.h>

#include <qpixmap.h>
#include <qbitmap.h>
#include <qpoint.h>


#ifdef USEKDE
#include <qptrlist.h>
#include <kdebug.h>
#else
#include <qlist.h>
#define QPtrList QList
#endif

#include "sprite.h"


Sprite::Sprite(QCanvas *canvas,QString filename, int frames) :
            QCanvasSprite(0,canvas)
{

  // Start idle
  setVelocity(0,0);
  
  // Load the graphics from file
  bool res = loadSprite(filename, frames);
  
  // We immediately show the sprite. In a non demo you might
  // do this later manually
  if (res) show();
}

Sprite::~Sprite()
{
  // Hide before destruction to make sure it is removed 
  // from all canvas
  hide();
}

bool Sprite::loadSprite(QString name, int frames)
{
  bool res_load= false;
   QPtrList<QPixmap> pixlist;
   pixlist.setAutoDelete(true);
   QPtrList<QPoint> hotlist;
   hotlist.setAutoDelete(true);

    for (int i=0;i<frames;i++)
   {
    // If only one frame shall be loaded do not append number!
    QString filename;
    QString no;
    no.sprintf("%d",i);
    if (frames == 1)
      {
      filename = name;
      }
    else
      {
      filename = name.arg(no);
      }

    QPixmap *pixmap = new QPixmap();
    res_load = pixmap->load(filename);
    // If only one frame and loading did not work, try with number
    // replacement
    if (!res_load && frames == 1)
    {
      filename = name.arg(no);
      res_load = pixmap->load(filename);
    }
     // kdDebug() << "Loading sprite " << filename << " = "<<res_load<<endl;
     // cout << "Loading sprite " << filename << " = "<<res_load<<endl;
    if (!res_load)
    {
       #ifdef USEKDE
       kdError() << "Could not load sprite "<< name << endl;
       #endif
       return false;
    }

   QPoint *point=new QPoint(pixmap->width()/2,pixmap->height()/2);
    hotlist.append(point);

    pixlist.append(pixmap);
   }

   QCanvasPixmapArray *pixmaparray=new QCanvasPixmapArray(pixlist,hotlist);
   setSequence(pixmaparray);
   return res_load;
}



// Provides the sprite animation and movement
void Sprite::advance(int phase)
{
  // Call parents function to do the move for us
  QCanvasSprite::advance(phase);
}


